package pro.litvinovg.libreoffice;

import com.sun.star.beans.XPropertySet;
import com.sun.star.container.NoSuchElementException;
import com.sun.star.container.XEnumeration;
import com.sun.star.container.XEnumerationAccess;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.table.XCell;
import com.sun.star.text.XText;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextRange;
import com.sun.star.text.XTextTable;
import com.sun.star.uno.UnoRuntime;


public class DocumentParserImpl implements DocumentParser{
	XText text = null;
	protected XTextRange documentEnd = null; 
	public DocumentParserImpl(XText text) {
		super();
		this.text = text;
	}
	public void parse() {
		parseText(text);
	}
	protected void parseText(XText curText) {
		XEnumerationAccess enumAccess = UnoRuntime.queryInterface(XEnumerationAccess.class, curText);
		XEnumeration textEnum = enumAccess.createEnumeration();
		while (textEnum.hasMoreElements()) {
			try {
				XTextContent textContent =  UnoRuntime.queryInterface(XTextContent.class, textEnum.nextElement());
				documentEnd = textContent.getAnchor().getEnd();
				if(isSupported(textContent, "com.sun.star.text.TextTable")) {
					parseTable(textContent);
				} else
				if (isSupported(textContent, "com.sun.star.style.ParagraphProperties")) {
					parseParagraph(textContent);
				} 
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (WrappedTargetException e) {
				e.printStackTrace();
			}
		}
	}

	private void parseTable(XTextContent textContent) {
		XTextTable table = UnoRuntime.queryInterface(XTextTable.class, textContent);
		String[] cellNames = table.getCellNames();
		for (String cellName : cellNames) {
			XCell cell = table.getCellByName(cellName);
			XText cellText = UnoRuntime.queryInterface(XText.class, cell);
			cellText.getText();
			parseText(cellText);
		}
	}

	private void parseParagraph(XTextContent textContent) {
		XPropertySet properties =  UnoRuntime.queryInterface(XPropertySet.class, textContent);
		paraProperties(properties, textContent);
    XEnumerationAccess enumAccess = UnoRuntime.queryInterface(XEnumerationAccess.class, textContent);
		XEnumeration paraEnum = enumAccess.createEnumeration();
		while (paraEnum.hasMoreElements()) {
			try {
				XPropertySet portionPoperties = UnoRuntime.queryInterface(com.sun.star.beans.XPropertySet.class, paraEnum.nextElement());
				portionProperties(portionPoperties, textContent);
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (WrappedTargetException e) {
				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	protected void portionProperties(XPropertySet portionPoperties, XTextContent textContent) {
	}
	protected void paraProperties(XPropertySet properties,XTextContent textContent) {
	}
	
	private boolean isSupported(Object object, String service) {
		XServiceInfo info = UnoRuntime.queryInterface(XServiceInfo.class, object);
		if (info == null) {
			return false;
		}
		return info.supportsService(service);
	}
}
